"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const inquirer_1 = __importDefault(require("inquirer"));
const chalk_1 = __importDefault(require("chalk"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const __1 = require("..");
exports.command = 'init';
exports.desc = 'Init basic zignis config file and directories';
exports.aliases = 'i';
exports.builder = function (yargs) {
    yargs.option('plugin', {
        default: false,
        alias: 'P',
        describe: 'plugin mode'
    });
    yargs.option('force', {
        alias: 'f',
        describe: 'force init!'
    });
    yargs.option('add', {
        default: false,
        alias: 'A',
        describe: 'add npm package to package.json dependencies'
    });
    yargs.option('add-dev', {
        default: false,
        alias: 'D',
        describe: 'add npm package to package.json devDependencies'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        let defaultZignisrc = argv.plugin
            ? {
                commandDir: 'src/commands',
                extendDir: 'src/extends',
                hookDir: 'src/hooks'
            }
            : {
                commandDir: 'bin/zignis/commands',
                pluginDir: 'bin/zignis/plugins',
                extendDir: 'bin/zignis/extends',
                scriptDir: 'bin/zignis/scripts',
                hookDir: 'bin/zignis/hooks'
            };
        let currentPath = path_1.default.resolve(process.cwd());
        try {
            const { override } = fs_1.default.existsSync(`${currentPath}/.zignisrc.json`) && !argv.force
                ? yield inquirer_1.default.prompt([
                    {
                        type: 'confirm',
                        name: 'override',
                        message: '.zignisrc.json exists, override?',
                        default: false
                    }
                ])
                : { override: true };
            if (override === false) {
                console.log(chalk_1.default.yellow('User aborted!'));
                return;
            }
            fs_1.default.writeFileSync(`${currentPath}/.zignisrc.json`, JSON.stringify(defaultZignisrc, null, 2));
            console.log(chalk_1.default.green('Default .zignisrc created!'));
            const dirs = Object.keys(defaultZignisrc);
            dirs.forEach(dir => {
                // @ts-ignore
                const loc = defaultZignisrc[dir];
                if (!fs_1.default.existsSync(`${currentPath}/${loc}`)) {
                    __1.Utils.exec(`mkdir -p ${currentPath}/${loc}`);
                    console.log(chalk_1.default.green(`${currentPath}/${loc} created!`));
                }
            });
            // add packages
            const addPackage = __1.Utils.parsePackageNames(argv.add);
            const addPackageDev = __1.Utils.parsePackageNames(argv.addDev);
            if (addPackage.length > 0) {
                if (argv.yarn) {
                    __1.Utils.exec(`yarn add ${addPackage.join(' ')}`);
                }
                else {
                    __1.Utils.exec(`npm install ${addPackage.join(' ')}`);
                }
            }
            if (addPackageDev.length > 0) {
                if (argv.yarn) {
                    __1.Utils.exec(`yarn add ${addPackageDev.join(' ')} -D`);
                }
                else {
                    __1.Utils.exec(`npm install ${addPackageDev.join(' ')} --save-dev`);
                }
            }
        }
        catch (e) {
            return __1.Utils.error(e.stack);
        }
    });
};
