"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const __1 = require("../..");
exports.command = 'command <name> [description]';
exports.desc = 'Generate a command template';
exports.builder = function (yargs) {
    yargs.option('extend', {
        default: false,
        alias: 'E',
        describe: 'generate command in extend directory, e.g. extend=zignis'
    });
    yargs.option('plugin', {
        default: false,
        alias: 'P',
        describe: 'generate command in plugin directory, e.g. plugin=xxx'
    });
    yargs.option('yield', {
        default: false,
        alias: 'Y',
        describe: 'use yield style for generator and promoise'
    });
};
exports.handler = function (argv) {
    let commandDir;
    if (argv.extend) {
        let extendName = argv.extend;
        if (extendName !== 'zignis' && extendName.indexOf('zignis-plugin-') !== 0) {
            extendName = `zignis-plugin-${extendName}`;
        }
        commandDir = `${argv.extendDir}/${extendName}/src/commands`;
    }
    else if (argv.plugin) {
        let pluginName = argv.plugin;
        if (pluginName.indexOf('zignis-plugin-') !== 0) {
            pluginName = `zignis-plugin-${pluginName}`;
        }
        commandDir = `${argv.pluginDir}/${pluginName}/src/commands`;
    }
    else {
        commandDir = argv.commandDir;
    }
    if (!commandDir) {
        __1.Utils.error(chalk_1.default.red('"commandDir" missing in config file!'));
    }
    const commandFilePath = path_1.default.resolve(commandDir, `${argv.name}.js`);
    const commandFileDir = path_1.default.dirname(commandFilePath);
    __1.Utils.fs.ensureDirSync(commandFileDir);
    if (fs_1.default.existsSync(commandFilePath)) {
        __1.Utils.error(chalk_1.default.red('Command file exist!'));
    }
    const name = argv.name.split('/').pop();
    let handerTpl;
    if (argv.yield) {
        handerTpl = `exports.handler = function (argv) {
  Utils.co(function * () {
    console.log('Start to draw your dream code!')
    Utils.info('Finished successfully!')
  }).catch(e => Utils.error(e.stack))
}`;
    }
    else {
        handerTpl = `exports.handler = async function (argv) {
  console.log('Start to draw your dream code!')
  Utils.info('Finished successfully!', true)
}`;
    }
    const code = `
const { Utils } = require('zignis')

exports.command = '${name}'
exports.desc = '${argv.description || name}'
// exports.aliases = ''

exports.builder = function (yargs) {
  // yargs.option('option', { default, describe, alias })
  // yargs.commandDir('${name}')
}

${handerTpl}
`;
    if (!fs_1.default.existsSync(commandFilePath)) {
        fs_1.default.writeFileSync(commandFilePath, code);
        console.log(chalk_1.default.green(`${commandFilePath} created!`));
    }
    // TODO: check zignis installed or not, if not ask if add, check yarn.lock, if exist use yarn or use npm
};
