"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const repl_history_1 = __importDefault(require("repl.history"));
const fs_1 = __importDefault(require("fs"));
const repl_1 = __importDefault(require("repl"));
const __1 = require("..");
exports.command = 'shell';
exports.desc = 'Shell for Zignis';
exports.aliases = 'sh';
function corepl(cli) {
    // @ts-ignore
    cli.eval = function coEval(cmd, context, filename, callback) {
        cmd = cmd.trim();
        if (['exit', 'quit', 'q'].includes(cmd.replace(/(^\s*)|(\s*$)/g, ''))) {
            __1.Utils.info('Bye.');
            process.exit(0);
        }
        if (cmd === '?') {
            console.log();
            __1.Utils.outputTable([
                ['quit', 'Quit the shell, alias: exit, q.'],
                ['prefix', 'You can change prefix option at any time.'],
                ['?', 'Show this help info.']
            ], 'Internal commands:');
            __1.Utils.info(`Current prefix: ${__1.Utils.chalk.yellow(context.argv.prefix || '[empty]')}`);
            console.log();
            return callback();
        }
        if (cmd.match(/^prefix[\s|=]+/)) {
            let prefix = __1.Utils.splitByChar(cmd, '=');
            prefix.shift();
            context.argv.prefix = prefix.join(' ').trim();
            __1.Utils.success(`Prefix has been changed to: ${context.argv.prefix || '[empty], so you can run any shell commands now.'}`);
            return callback();
        }
        if (`${context.argv.prefix} ${cmd}`.match(/^zignis\s+shell\s+/) ||
            `${context.argv.prefix} ${cmd}`.match(/^zignis\s+sh\s+/)) {
            __1.Utils.warn('Recursive call shell not allowed!');
            return callback();
        }
        if (`${context.argv.prefix} ${cmd}`.match(/^zignis\s+/)) {
            cmd = `${context.argv.prefix} ${cmd} --exec-mode`;
        }
        else {
            cmd = `${context.argv.prefix} ${cmd}`;
        }
        try {
            cmd && __1.Utils.exec(cmd);
        }
        catch (e) {
            if (context.argv.debug) {
                __1.Utils.error(e.stack, false);
            }
            else {
                __1.Utils.error(e.message, false);
            }
        }
        return callback();
    };
    return cli;
}
function openRepl(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const r = repl_1.default.start({
            prompt: context.argv.prompt,
            completer: () => {
                // TODO: implments auto completion in REPL by Shell suggestions.
                // For now, it is just for disabling Node completion.
                return [];
            }
        });
        const zignisHome = process.env.HOME + '/.zignis';
        if (!fs_1.default.existsSync(zignisHome)) {
            __1.Utils.exec(`mkdir -p ${zignisHome}`);
        }
        repl_history_1.default(r, `${zignisHome}/.zignis_shell_history`);
        // @ts-ignore
        // context即为REPL中的上下文环境
        r.context = Object.assign(r.context, context);
        corepl(r);
    });
}
exports.builder = function (yargs) {
    yargs.option('prompt', {
        default: '$ ',
        describe: 'Prompt for input.'
    });
    yargs.option('prefix', {
        default: 'zignis',
        describe: 'Make input zignis command a little bit faster.'
    });
    yargs.option('debug', {
        default: false,
        describe: 'Debug mode, show error stack'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            let context = { argv };
            return yield openRepl(context);
        }
        catch (e) {
            __1.Utils.error(e.stack);
        }
    });
};
