"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
exports.fs = fs_extra_1.default;
const glob_1 = __importDefault(require("glob"));
exports.glob = glob_1.default;
const table_1 = require("table");
exports.table = table_1.table;
const find_up_1 = __importDefault(require("find-up"));
exports.findUp = find_up_1.default;
const lodash_1 = __importDefault(require("lodash"));
exports._ = lodash_1.default;
const json_colorizer_1 = __importDefault(require("json-colorizer"));
exports.colorize = json_colorizer_1.default;
const json_stringify_pretty_compact_1 = __importDefault(require("json-stringify-pretty-compact"));
exports.stringify = json_stringify_pretty_compact_1.default;
const chalk_1 = __importDefault(require("chalk"));
exports.chalk = chalk_1.default;
const randomatic_1 = __importDefault(require("randomatic"));
exports.randomatic = randomatic_1.default;
const dayjs_1 = __importDefault(require("dayjs"));
exports.day = dayjs_1.default;
const co_1 = __importDefault(require("co"));
exports.co = co_1.default;
const shelljs_1 = __importDefault(require("shelljs"));
exports.shell = shelljs_1.default;
const debug_1 = __importDefault(require("debug"));
exports.debug = debug_1.default;
const inquirer_1 = __importDefault(require("inquirer"));
exports.inquirer = inquirer_1.default;
const inquirer_autocomplete_prompt_1 = __importDefault(require("inquirer-autocomplete-prompt"));
const fuzzy_1 = __importDefault(require("fuzzy"));
exports.fuzzy = fuzzy_1.default;
const child_process_1 = require("child_process");
const node_object_hash_1 = __importDefault(require("node-object-hash"));
const node_emoji_1 = __importDefault(require("node-emoji"));
exports.emoji = node_emoji_1.default;
const dumper_js_1 = require("dumper.js");
exports.dd = dumper_js_1.dd;
exports.dump = dumper_js_1.dump;
const get_stdin_1 = __importDefault(require("get-stdin"));
exports.getStdin = get_stdin_1.default;
const node_cache_1 = __importDefault(require("node-cache"));
exports.NodeCache = node_cache_1.default;
inquirer_1.default.registerPrompt('autocomplete', inquirer_autocomplete_prompt_1.default);
const { hash } = node_object_hash_1.default({ sort: true });
const debugCore = debug_1.default('zignis-core');
let cachedInstance;
/**
 * Get Zignis internal cache instance
 * @returns {NodeCache}
 */
const getInternalCache = function () {
    if (!cachedInstance) {
        cachedInstance = new node_cache_1.default({
            useClones: false
        });
    }
    return cachedInstance;
};
exports.getInternalCache = getInternalCache;
cachedInstance = getInternalCache();
/**
 * Get Zignis cache instance by namespace
 * @param {string} namespace
 * @returns {NodeCache}
 */
const getCache = function (namespace) {
    if (!namespace) {
        throw Error('Namespace is necessary.');
    }
    let cachedNamespaceInstances;
    cachedNamespaceInstances = cachedInstance.get('cachedNamespaceInstances');
    if (!cachedNamespaceInstances) {
        cachedNamespaceInstances = {};
    }
    if (!cachedNamespaceInstances[namespace]) {
        cachedNamespaceInstances[namespace] = new node_cache_1.default({
            useClones: false
        });
        cachedInstance.set('cachedNamespaceInstances', cachedNamespaceInstances);
    }
    return cachedNamespaceInstances[namespace];
};
exports.getCache = getCache;
/**
 * Run hook in all valid plugins and return the combined results.
 * Plugins implement hook in `module.exports`, could be generator function or promise function or non-function
 * For non-function, it will be used as hook data directly, likely to be returned by function
 * @example
 * const hookReturn = await Utils.invokeHook('hook')
 * @param {string} hook Hook name, suggest plugin defined hook include a prefix, e.g. `zhike:hook`
 * @param {string} options Options
 * @param {string} options.mode Hook mode, could be `assign`, `merge`, `push`, `replace`, `group`, default is assign.
 * @param {bool} options.useCache If or not use cached hook result
 * @param {array} options.include set plugins to be used in invoking
 * @param {array} options.exclude set plugins not to be used in invoking, same ones options.exclude take precedence
 * @param {array} options.opts opts will be sent to hook implementation
 */
const invokeHook = function (hook, options = { mode: 'assign' }) {
    return __awaiter(this, void 0, void 0, function* () {
        const invokedHookCache = cachedInstance.get('invokedHookCache') || {};
        hook = `hook_${hook}`;
        options = Object.assign({
            mode: 'assign',
            useCache: false,
            include: [],
            exclude: [],
            opts: {}
        }, options);
        try {
            const cacheKey = `${hook}:${hash(options)}`;
            if (options.useCache && invokedHookCache[cacheKey]) {
                return invokedHookCache[cacheKey];
            }
            // Make Zignis core supporting hook invocation
            const plugins = Object.assign({}, {
                zignis: path_1.default.resolve(__dirname, '../../')
            }, getAllPluginsMapping());
            // Make Application supporting hook invocation
            const appConfig = getApplicationConfig();
            if (appConfig && appConfig.name !== 'zignis' && !plugins[appConfig.name] && appConfig.applicationDir) {
                plugins['application'] = appConfig.applicationDir;
            }
            let pluginsReturn;
            switch (options.mode) {
                case 'push':
                    pluginsReturn = [];
                    break;
                case 'replace':
                    pluginsReturn = '';
                    break;
                case 'group':
                case 'assign':
                case 'merge':
                default:
                    pluginsReturn = {};
                    break;
            }
            for (let i = 0, length = Object.keys(plugins).length; i < length; i++) {
                let plugin = Object.keys(plugins)[i];
                if (lodash_1.default.isArray(options.include) && options.include.length > 0 && options.include.indexOf(plugin) === -1) {
                    continue;
                }
                if (lodash_1.default.isArray(options.exclude) && options.exclude.length > 0 && options.exclude.indexOf(plugin) > -1) {
                    continue;
                }
                try {
                    let pluginEntry = 'index.js';
                    if (fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], 'package.json'))) {
                        const pkgConfig = require(path_1.default.resolve(plugins[plugin], 'package.json'));
                        if (pkgConfig.main) {
                            pluginEntry = pkgConfig.main;
                        }
                    }
                    // hookDir
                    if (fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], '.zignisrc.json'))) {
                        const zignisConfig = require(path_1.default.resolve(plugins[plugin], '.zignisrc.json'));
                        if (zignisConfig.hookDir && fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], zignisConfig.hookDir, 'index.js'))) {
                            pluginEntry = path_1.default.join(zignisConfig.hookDir, 'index.js');
                        }
                    }
                    // For application, do not accept default index.js
                    if (plugin === 'application' && pluginEntry === 'index.js') {
                        continue;
                    }
                    if (fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], pluginEntry))) {
                        debugCore(`zignis invoke hook: ${hook}, plugin: ${plugin}`);
                        const loadedPlugin = require(path_1.default.resolve(plugins[plugin], pluginEntry));
                        if (loadedPlugin[hook]) {
                            let pluginReturn;
                            if (lodash_1.default.isFunction(loadedPlugin[hook])) {
                                pluginReturn = (yield loadedPlugin[hook](options.opts, pluginsReturn)) || {};
                            }
                            else {
                                pluginReturn = loadedPlugin[hook];
                            }
                            switch (options.mode) {
                                case 'group':
                                    pluginReturn = pluginReturn || {};
                                    pluginsReturn[plugin] = pluginReturn;
                                    break;
                                case 'push':
                                    pluginsReturn.push(pluginReturn);
                                    break;
                                case 'replace':
                                    pluginsReturn = pluginReturn;
                                    break;
                                case 'merge':
                                    pluginReturn = pluginReturn || {};
                                    pluginsReturn = lodash_1.default.merge(pluginsReturn, pluginReturn);
                                    break;
                                case 'assign':
                                default:
                                    pluginReturn = pluginReturn || {};
                                    pluginsReturn = Object.assign(pluginsReturn, pluginReturn);
                                    break;
                            }
                        }
                    }
                }
                catch (e) {
                    if (!e.code || e.code !== 'MODULE_NOT_FOUND') {
                        throw new Error(e.stack);
                    }
                    else {
                        error(e.message, false);
                    }
                }
            }
            invokedHookCache[cacheKey] = pluginsReturn;
            cachedInstance.set('invokedHookCache', invokedHookCache);
            return pluginsReturn;
        }
        catch (e) {
            throw new Error(e.stack);
        }
    });
};
exports.invokeHook = invokeHook;
/**
 * Extend command's sub command, it give other plugins an opportunity to extend it's sub command.
 * So if you want other plugins to extend your sub commands, you can use this util function to replace default `yargs.commandDir`
 * @example
 * exports.builder = function (yargs) {
 *   // The first param could be a/b/c if you want to extend subcommand's subcommand
 *   Utils.extendSubCommand('make', 'zignis', yargs, __dirname)
 * }
 * @param {String} command Current command name.
 * @param {String} module Current plugin name.
 * @param {Object} yargs Yargs reference.
 * @param {String} basePath Often set to `__dirname`.
 */
const extendSubCommand = function (command, module, yargs, basePath) {
    const plugins = getAllPluginsMapping();
    const config = getCombinedConfig();
    // load default commands
    const currentCommand = command.split('/').pop();
    if (currentCommand && fs_extra_1.default.existsSync(path_1.default.resolve(basePath, currentCommand))) {
        yargs.commandDir(path_1.default.resolve(basePath, currentCommand));
    }
    // Load plugin commands
    if (plugins) {
        Object.keys(plugins).map(function (plugin) {
            if (config.pluginConfigs[plugin] && config.pluginConfigs[plugin].extendDir) {
                if (fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], `${config.pluginConfigs[plugin].extendDir}/${module}/src/commands`, command))) {
                    yargs.commandDir(path_1.default.resolve(plugins[plugin], `${config.pluginConfigs[plugin].extendDir}/${module}/src/commands`, command));
                }
            }
        });
    }
    // Load application commands
    if (config.extendDir &&
        fs_extra_1.default.existsSync(path_1.default.resolve(process.cwd(), `${config.extendDir}/${module}/src/commands`, command))) {
        yargs.commandDir(path_1.default.resolve(process.cwd(), `${config.extendDir}/${module}/src/commands`, command));
    }
};
exports.extendSubCommand = extendSubCommand;
/**
 * Get all plugins path mapping.
 * Same name plugins would be overriden orderly.
 * This function also influence final valid commands and configs.
 */
const getAllPluginsMapping = function () {
    let argv = cachedInstance.get('argv') || {};
    let plugins = cachedInstance.get('plugins') || {};
    if (lodash_1.default.isEmpty(plugins)) {
        plugins = {};
        // process core plugins
        glob_1.default
            .sync('zignis-plugin-*', {
            cwd: path_1.default.resolve(__dirname, '../plugins')
        })
            .map(function (plugin) {
            plugins[plugin] = path_1.default.resolve(__dirname, '../plugins', plugin);
        });
        if (!argv.disableGlobalPlugin) {
            // process core same directory plugins
            glob_1.default
                .sync('zignis-plugin-*', {
                cwd: path_1.default.resolve(__dirname, '../../../')
            })
                .map(function (plugin) {
                plugins[plugin] = path_1.default.resolve(__dirname, '../../../', plugin);
            });
            // process core same directory npm plugins
            glob_1.default
                .sync('@*/zignis-plugin-*', {
                cwd: path_1.default.resolve(__dirname, '../../../')
            })
                .map(function (plugin) {
                if (fs_extra_1.default.existsSync(path_1.default.resolve(__dirname, '../../../', plugin))) {
                    plugins[plugin] = path_1.default.resolve(__dirname, '../../../', plugin);
                }
            });
        }
        if (process.env.HOME && !argv.disableHomePlugin) {
            // process home npm plugins
            glob_1.default
                .sync('zignis-plugin-*', {
                cwd: path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules')
            })
                .map(function (plugin) {
                if (process.env.HOME && fs_extra_1.default.existsSync(path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules', plugin))) {
                    plugins[plugin] = path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules', plugin);
                }
            });
            // process home npm scope plugins
            glob_1.default
                .sync('@*/zignis-plugin-*', {
                cwd: path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules')
            })
                .map(function (plugin) {
                if (process.env.HOME && fs_extra_1.default.existsSync(path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules', plugin))) {
                    plugins[plugin] = path_1.default.resolve(process.env.HOME, '.zignis', 'node_modules', plugin);
                }
            });
        }
        // process cwd npm plugins
        glob_1.default
            .sync('zignis-plugin-*', {
            cwd: path_1.default.resolve(process.cwd(), 'node_modules')
        })
            .map(function (plugin) {
            if (fs_extra_1.default.existsSync(path_1.default.resolve(process.cwd(), 'node_modules', plugin))) {
                plugins[plugin] = path_1.default.resolve(process.cwd(), 'node_modules', plugin);
            }
        });
        // process cwd npm scope plugins
        glob_1.default
            .sync('@*/zignis-plugin-*', {
            cwd: path_1.default.resolve(process.cwd(), 'node_modules')
        })
            .map(function (plugin) {
            if (fs_extra_1.default.existsSync(path_1.default.resolve(process.cwd(), 'node_modules', plugin))) {
                plugins[plugin] = path_1.default.resolve(process.cwd(), 'node_modules', plugin);
            }
        });
        const config = getApplicationConfig();
        if (fs_extra_1.default.existsSync(config.pluginDir)) {
            // process local plugins
            glob_1.default
                .sync('zignis-plugin-*', {
                cwd: path_1.default.resolve(process.cwd(), config.pluginDir)
            })
                .map(function (plugin) {
                if (fs_extra_1.default.existsSync(path_1.default.resolve(process.cwd(), config.pluginDir, plugin))) {
                    plugins[plugin] = path_1.default.resolve(process.cwd(), config.pluginDir, plugin);
                }
            });
        }
        // process plugin project
        if (fs_extra_1.default.existsSync(path_1.default.resolve(process.cwd(), 'package.json'))) {
            const pkgConfig = require(path_1.default.resolve(process.cwd(), 'package.json'));
            if (pkgConfig.name && /^(@[^/]+\/)?zignis-plugin-/.test(pkgConfig.name)) {
                plugins[pkgConfig.name] = path_1.default.resolve(process.cwd());
            }
        }
        cachedInstance.set('plugins', plugins);
    }
    return plugins;
};
exports.getAllPluginsMapping = getAllPluginsMapping;
/**
 * Get application zignis config only.
 */
const getApplicationConfig = function (cwd = undefined) {
    try {
        const configPath = find_up_1.default.sync(['.zignisrc.json'], {
            cwd
        });
        let applicationConfig = configPath ? require(configPath) : {};
        if (configPath) {
            applicationConfig.applicationDir = path_1.default.dirname(configPath);
            if (fs_extra_1.default.existsSync(path_1.default.resolve(applicationConfig.applicationDir, 'package.json'))) {
                applicationConfig = Object.assign({}, applicationConfig, require(path_1.default.resolve(applicationConfig.applicationDir, 'package.json')));
            }
        }
        return applicationConfig;
    }
    catch (e) {
        error(`Application .zignisrc.json can not be parsed!`);
    }
};
exports.getApplicationConfig = getApplicationConfig;
/**
 * Get commbined config from whole environment.
 */
const getCombinedConfig = function () {
    let combinedConfig = cachedInstance.get('combinedConfig') || {};
    let pluginConfigs = {};
    if (lodash_1.default.isEmpty(combinedConfig)) {
        if (process.env.HOME && fs_extra_1.default.existsSync(path_1.default.resolve(process.env.HOME, '.zignis', '.zignisrc.json'))) {
            combinedConfig = require(path_1.default.resolve(process.env.HOME, '.zignis', '.zignisrc.json'));
        }
        else {
            combinedConfig = {};
        }
        const plugins = getAllPluginsMapping();
        Object.keys(plugins).map(plugin => {
            if (fs_extra_1.default.existsSync(path_1.default.resolve(plugins[plugin], '.zignisrc.json'))) {
                const pluginConfig = require(path_1.default.resolve(plugins[plugin], '.zignisrc.json'));
                combinedConfig = lodash_1.default.merge(combinedConfig, pluginConfig);
                pluginConfigs[plugin] = pluginConfig;
            }
        });
        const configPath = find_up_1.default.sync(['.zignisrc.json']);
        let rcConfig = configPath ? require(configPath) : {};
        combinedConfig = lodash_1.default.merge(combinedConfig, rcConfig);
        combinedConfig.pluginConfigs = pluginConfigs;
        cachedInstance.set('combinedConfig', combinedConfig);
    }
    return combinedConfig || {};
};
exports.getCombinedConfig = getCombinedConfig;
/**
 * Print message with format and color.
 * @param {mix} message Message to log
 */
const log = function (message) {
    if (lodash_1.default.isArray(message) || lodash_1.default.isObject(message)) {
        console.log(json_colorizer_1.default(json_stringify_pretty_compact_1.default(message)));
    }
    else {
        console.log(message);
    }
};
exports.log = log;
/**
 * Print error message, and exit process.
 * @param {mix} message Error message to log
 * @param {string} label Error log label
 * @param {integer} errorCode Error code
 */
const error = function (message, exit = true, errorCode = 1) {
    message = lodash_1.default.isString(message) ? { message } : message;
    console.log(node_emoji_1.default.get(message.emoji || 'x'), ' ', chalk_1.default.red(message.message));
    if (exit) {
        process.exit(errorCode);
    }
};
exports.error = error;
/**
 * Print warn message with yellow color.
 * @param {mix} message Error message to log
 */
const warn = function (message, exit = false, errorCode = 0) {
    message = lodash_1.default.isString(message) ? { message } : message;
    console.log(chalk_1.default.yellow(message.message));
    if (exit) {
        process.exit(errorCode);
    }
};
exports.warn = warn;
/**
 * Print info message with green color.
 * @param {mix} message Error message to log
 */
const info = function (message, exit = false, errorCode = 0) {
    message = lodash_1.default.isString(message) ? { message } : message;
    console.log(chalk_1.default.cyan(message.message));
    if (exit) {
        process.exit(errorCode);
    }
};
exports.info = info;
/**
 * Print success message with green color.
 * @param {mix} message Error message to log
 */
const success = function (message, exit = false, errorCode = 0) {
    message = lodash_1.default.isString(message) ? { message } : message;
    console.log(chalk_1.default.green(message.message));
    if (exit) {
        process.exit(errorCode);
    }
};
exports.success = success;
/**
 * Compute md5.
 * @param {string} s
 */
const md5 = function (s) {
    return crypto_1.default
        .createHash('md5')
        .update(s, 'utf8')
        .digest('hex');
};
exports.md5 = md5;
/**
 * Split input by comma and blank.
 * @example
 * const = Utils.splitComma('a, b , c,d')
 * @param {string} input
 * @returns {array} input separated by comma
 */
const splitComma = function (input) {
    return splitByChar(input, ',');
};
exports.splitComma = splitComma;
/**
 * Split input by a specific char and blank.
 * @example
 * const = Utils.splitByChar('a, b , c=d', '=')
 * @param {string} input
 * @returns {array} input separated by comma
 */
const splitByChar = function (input, char) {
    const exp = new RegExp(char, 'g');
    return input.replace(exp, ' ').split(/\s+/);
};
exports.splitByChar = splitByChar;
/**
 * Print a simple table.
 * A table style for `zignis status`, if you don't like this style, can use Utils.table
 * @param {array} columns Table columns
 * @param {string} caption Table caption
 * @param {object} borderOptions Border options
 */
const outputTable = function (columns, caption, borderOptions = {}) {
    // table config
    const config = {
        drawHorizontalLine: () => {
            return false;
        },
        columnDefault: {
            paddingLeft: 2,
            paddingRight: 1
        },
        border: Object.assign(table_1.getBorderCharacters(`void`), { bodyJoin: `:` }, borderOptions)
    };
    if (caption) {
        info(caption);
    }
    console.log(table_1.table(columns, config));
};
exports.outputTable = outputTable;
/**
 * Parse packages from yargs option
 * @param {*} input yarns option input, could be string or array
 * @returns {array} Package list
 */
const parsePackageNames = function (input) {
    if (lodash_1.default.isString(input)) {
        return splitComma(input);
    }
    if (lodash_1.default.isArray(input)) {
        return lodash_1.default.flatten(input.map(item => splitComma(item)));
    }
    return [];
};
exports.parsePackageNames = parsePackageNames;
/**
 * Load any package's package.json
 * @param {string} pkg package name
 * @param {array} paths search paths
 */
const loadPackageInfo = function (pkg = undefined, paths = []) {
    const packagePath = find_up_1.default.sync('package.json', {
        cwd: pkg ? path_1.default.dirname(require.resolve(pkg, { paths })) : process.cwd()
    });
    return packagePath ? require(packagePath) : {};
};
exports.loadPackageInfo = loadPackageInfo;
/**
 * Load core package.json
 */
const loadCorePackageInfo = function () {
    const packagePath = find_up_1.default.sync('package.json', {
        cwd: path_1.default.resolve('../../', __dirname)
    });
    return packagePath ? require(packagePath) : {};
};
exports.loadCorePackageInfo = loadCorePackageInfo;
/**
 * Execute command, because npm install running info can not be catched by shelljs, temporarily use this one
 * @param {string} command Command to exec
 * @param {object} options Options stdio default is [0, 1, 2]
 */
const exec = function (command, options = {}) {
    if (!options.stdio) {
        options.stdio = [0, 1, 2];
    }
    return child_process_1.execSync(command, options);
};
exports.exec = exec;
/**
 * Sleep a while of ms
 * @param {integer} ms
 */
const sleep = (ms) => new Promise(resolve => setTimeout(resolve, ms));
exports.sleep = sleep;
const delay = sleep;
exports.delay = delay;
