#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const update_notifier_1 = __importDefault(require("update-notifier"));
// import pkg from '../package.json'
const yargs_1 = __importDefault(require("yargs"));
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const debug = _1.Utils.debug('zignis-core');
debug('zignis started');
const pkg = _1.Utils.loadCorePackageInfo();
update_notifier_1.default({ pkg, updateCheckInterval: 1000 * 60 * 60 * 24 * 7 }).notify({
    defer: false,
    isGlobal: true
});
debug('zignis update notifier');
let parsedArgv = yargs_parser_1.default(process.argv.slice(2));
const config = _1.Utils.getCombinedConfig();
yargs_1.default.config(config);
parsedArgv = _1.Utils._.merge(parsedArgv, config);
const cache = _1.Utils.getInternalCache();
cache.set('argv', parsedArgv);
debug('zignis set cache argv');
const plugins = _1.Utils.getAllPluginsMapping();
debug('zignis get plugins');
const packageConfig = _1.Utils.loadPackageInfo();
if (!parsedArgv.disableCoreCommand) {
    // Load local commands
    if (packageConfig.name !== 'zignis') {
        yargs_1.default.commandDir('commands');
    }
    else if (config.commandDir && fs_1.default.existsSync(path_1.default.resolve(process.cwd(), config.commandDir))) {
        yargs_1.default.commandDir(path_1.default.resolve(process.cwd(), config.commandDir));
    }
}
// Load plugin commands
if (plugins) {
    Object.keys(plugins).map(function (plugin) {
        if (fs_1.default.existsSync(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir))) {
            yargs_1.default.commandDir(path_1.default.resolve(plugins[plugin], config.pluginConfigs[plugin].commandDir));
        }
    });
}
// Load application commands
if (packageConfig.name !== 'zignis' &&
    config.commandDir &&
    fs_1.default.existsSync(path_1.default.resolve(process.cwd(), config.commandDir))) {
    yargs_1.default.commandDir(path_1.default.resolve(process.cwd(), config.commandDir));
}
debug('zignis set commands');
(() => __awaiter(this, void 0, void 0, function* () {
    try {
        if (!parsedArgv.getYargsCompletions) {
            debug('zignis before command hook');
            let beforeHooks = yield _1.Utils.invokeHook('beforeCommand');
            Object.keys(beforeHooks).map(function (hook) {
                beforeHooks[hook](parsedArgv, yargs_1.default);
            });
        }
        if (!parsedArgv.scriptName) {
            yargs_1.default.option('script-name', {
                default: false,
                describe: 'Rename script name.'
            });
        }
        else {
            yargs_1.default.scriptName(parsedArgv.scriptName);
        }
        if (!parsedArgv.disableCoreCommand) {
            yargs_1.default.option('disable-core-command', {
                alias: 'disable-core',
                describe: 'Disable core commands.'
            });
        }
        if (!parsedArgv.disableGlobalPlugin) {
            yargs_1.default.option('disable-global-plugin', {
                alias: 'disable-global-plugins',
                describe: 'Disable global plugins.'
            });
        }
        if (!parsedArgv.disableHomePlugin) {
            yargs_1.default.option('disable-home-plugin', {
                alias: 'disable-home-plugins',
                describe: 'Disable home plugins.'
            });
        }
        // eslint-disable-next-line
        yargs_1.default
            .help()
            .completion('completion', 'Generate completion script')
            .alias('h', 'help')
            .exitProcess(false)
            .recommendCommands()
            .epilog('Find more information at https://zignis.js.org')
            .wrap(Math.min(120, yargs_1.default.terminalWidth())).argv;
        if (!parsedArgv.getYargsCompletions) {
            let afterHooks = yield _1.Utils.invokeHook('afterCommand');
            Object.keys(afterHooks).map(function (hook) {
                afterHooks[hook](parsedArgv, yargs_1.default);
            });
            debug('zignis after command hook');
        }
    }
    catch (e) {
        if (!e.name || e.name !== 'YError') {
            _1.Utils.error(e.stack);
        }
    }
}))();
